program StructoApp;

{---------------------------------------------------------------------
Example application for the custom control Structo (Structo.DLL) of
the SMWCC 1.0 Custom Control Pack

Copyright (C) by Sebastian Modersohn
---------------------------------------------------------------------}

{$R StructAp}

uses WinTypes, WinProcs, OWindows, ODialogs, Strings, BWCC,
     {the import unit for the button}
     Structo, StructCo;

{for loading the DLL}
const DLLName        ='STRUCTOS.DLL';
      em_DLLNotFound =1;
{some Id's}
      id_Ex1         =201;
      id_Ex2         =202;

type
  PStructoWindow = ^TStructoWindow;
  TStructoWindow = object(TDlgWindow)
    Ex1, Ex2: PStructo;
    constructor Init(AParent: PWindowsObject; AName: PChar);

    procedure WMMeasureItem(var Msg: TMessage);
      virtual wm_first + wm_MeasureItem;
    procedure WMDrawItem(var Msg: TMessage);
      virtual wm_first + wm_DrawItem;

    procedure Ok(var Msg: TMessage);
      virtual id_First + id_OK;
    procedure Help(var Msg: TMessage);
      virtual id_First + idHelp;
  end;

{App that loads and frees the DLL and initializes the main window}
  PStructoApp = ^TStructoApp;
  TStructoApp = object(TApplication)
    Lib: THandle;
    constructor Init(AName: PChar);
    destructor Done; virtual;
    procedure InitMainWindow; virtual;
    procedure Error(ErrorCode: Integer); virtual;
  end;

{ TStructoWindow }

constructor TStructoWindow.Init(AParent: PWindowsObject; AName: PChar);
begin
  inherited Init(AParent, AName);
  {init the two structos}
  Ex1:=New(PStructo, InitResource(@Self, id_Ex1));
  Ex2:=New(PStructo, InitResource(@Self, id_Ex2));
end;

procedure TStructoWindow.WMMeasureItem(var Msg: TMessage);
var MeasureItemStruct: PMeasureItemStruct;
begin
  MeasureItemStruct:=PMeasureItemStruct(Msg.LParam);
  case MeasureItemStruct^.CtlID of
    id_Ex1, id_Ex2:
      begin
        MeasureItemStruct^.itemHeight := 200;
        MeasureItemStruct^.itemWidth  := 200;
      end;
  end; {case}
end;

procedure tStructoWindow.WMDrawItem(var Msg: TMessage);
var DrawItemStruct:PDrawItemStruct;
    Strg: array[0..4] of Char;

procedure PaintStructo;
var x, y: Byte;
begin
  SetTextColor(DrawItemStruct^.hDC, TColorRef(RGB(0, 0, 0)));
  SetBkColor(DrawItemStruct^.hDC, TColorRef(RGB(192, 192, 192)));

  for x:=1 to 10 do
    for y:=1 to 10 do
    begin
      Str(X+Y, Strg);
      TextOut(DrawItemStruct^.hDC, (x-1)*20, (y-1)*20, Strg, StrLen(Strg));
    end;
end;

begin
  DrawItemStruct := PDrawItemStruct(Msg.LParam);
  case DrawItemStruct^.CtlID of
    id_Ex1, id_Ex2:
      begin
        if DrawItemStruct^.ItemAction=oda_DrawEntire then
            PaintStructo else DefWndProc(Msg);
      end else DefWndProc(Msg);
  end; {case}
end;

procedure TStructoWindow.Ok(var Msg: TMessage);
begin
  CloseWindow;
end;

procedure TStructoWindow.Help(var Msg: TMessage);
begin
  WinHelp(HWindow, HelpFile, Help_Context, 300);
end;

{ TStructoApp }

constructor TStructoApp.Init(AName: PChar);
begin
  {Load the library}
  Lib := LoadLibrary(DLLName);
  {If it wasn't loaded succesfully, then call error with em_DLLNotFound}
  Lib := LoadLibrary(DLLName);
  if Lib < 32 then
    Status := em_DLLNotFound
  else
    inherited Init(AName);
end;

destructor TStructoApp.Done;
begin
  inherited Done;
  {Free the library}
  FreeLibrary(Lib);
end;

procedure TStructoApp.InitMainWindow;
begin
  MainWindow := New(PStructoWindow, Init(nil, MakeIntResource(100)));
end;

procedure TStructoApp.Error(ErrorCode: Integer);
begin
  case ErrorCode of
    em_DLLNotFound:
      begin
        {if the dll could not be loaded display the message and fail the app}
        MessageBox(0, DLLName + ' not found. Please make sure that Windows will found this file!', 'Hard Error',
          mb_Ok or mb_IconStop);
        Halt(ErrorCode);
      end;
  else
    inherited Error(ErrorCode);
  end;
end;

var
  App: TStructoApp;

begin
  App.Init('Structo Demo');
  App.Run;
  App.Done;
end.
